/**************************************************************************
 *                                                                        *
 * Interface file to swig                                                 *
 *                                                                        *
 **************************************************************************/

%module arcbindings

%{

#include "DateTime.h"
#include "MdsQuery.h"
#include "CertInfo.h"
#include "LdapQuery.h"
#include <globus_common.h>
#include <globus_rsl.h>
#include "MdsQuery.h"
#include "Target.h"
#include "Xrsl.h"
#include "time.h"
#include "Preferences.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Environment.h"

#include "wrappers.h"

%}

%include "std_string.i"
%include "std_vector.i"
%include "std_map.i"
%include "wrappers.h"        // Wrapper functions defined here

%template(string_vector) std::vector<std::string>;
%template(giis_vector) std::vector< Giis >;
%template(job_vector) std::vector< Job >;
%template(cluster_vector) std::vector< Cluster >;
%template(queue_vector) std::vector < Queue >;
%template(target_vector) std::vector < Target >;

// These functions does not need to be wrapped
extern void ActivateGlobus();
extern void DeactivateGlobus();
extern const std::string TimeStamp ();

/* MdsQuery.h */
class Job {

 public:
  Job(){};
  Job (const string & id_);
  ~Job () {};
  const string & GetId () const;
  const string & GetStatus () const;
  const string & GetName () const;
  const string & GetErrors() const;
  const string & GetStdout () const;
  const string & GetStderr () const;
  const string & GetGmlog () const;
  int GetReqCpuTime () const;
  int GetCpuCount () const;
  void SetAttr (const string & attr, const string & value);
  void Print (ostream & s = cout) const;
  void PrintShort (ostream & s = cout) const;
  bool IsStatus (const vector <string> & stat);
 private:
  string id;
  string owner;
  string execcluster;
  string execqueue;
  string stdout;
  string stderr;
  string stdin;
  string gmlog;
  int reqcput;
  string status;
  int queuerank;
  string lrmscomment;
  string submissionui;
  string submissiontime;
  int usedcputime;
  int usedwalltime;
  string erasetime;
  int usedmem;
  string errors;
  string jobname;
  vector <Environment> runtimeenv;
  int cpucount;
  vector <string> execnodes;
  string clientsoftware;
  string proxyexptime;
};


class Queue {

 public:
  Queue() {};
  ~Queue () {};
  const string & GetName () const;
  const string & GetStatus () const;
  int GetRunning () const;
  int GetQueued () const;
  int GetMaxRunning () const;
  int GetMaxQueuable () const;
  int GetMaxCpuTime () const;
  int GetMinCpuTime () const;
  int GetDefaultCpuTime () const;
  int GetTotalCpus () const;
  int GetNodeMemory () const;
  const string & GetArchitecture () const;
  int GetFrequency () const;
  int GetUserFreeCpus (int minutes = 0) const;
  long long int GetUserDiskSpace () const;
  void ClaimUserDiskSpace (long long int);
  const Environment * FindEnvironment (const string & attr,
                       const EnvironmentTest & envtest) const;
  void SetAttr (const string & attr, const string & value);
  void Print (ostream & s = cout) const;
  void PrintShort (ostream & s = cout) const;
  void Accept (int minutes, int count);
 private:
  string name;
  string status;
  int running;
  int queued;
  int maxrunning;
  int maxqueuable;
  int maxuserrun;
  int maxcputime;
  int mincputime;
  int defaultcputime;
  string schedpolicy;
  int totalcpus;
  string nodecpu;
  int nodemem;
  string arch;
  vector <Environment> opsys;
  int gridrunning;
  int gridqueued;
  vector <Environment> runtimeenv;
  vector <Environment> middleware;
  map <int, int> userfreecpus;
  long long int userdiskspace;
  int userqueuelength;
 public:
  vector <Job> jobs;
};

class Cluster {

 public:
  Cluster() {};
  ~Cluster () {};
  const string & GetName () const;
  const string   GetLongName () const;
  const string & GetContact () const;
  const string & GetArchitecture () const;
  int GetFrequency () const;
  int GetNodeMemory () const;
  int GetTotalCpus () const;
  int GetQueued () const;
  long long int GetCacheFree () const;
  void ClaimCache (long long int size);
  long long int GetSessionDirFree () const;
  void ClaimSessionDir (long long int size);
  const Environment * FindEnvironment (const string & attr,
                       const EnvironmentTest & envtest) const;
  bool MatchLocalSe (const string & location) const;
  bool HaveNodeAccess (const string & attr) const;
  bool HaveKeys () const;
  void SetAttr (const string & attr, const string & value);
  void Print (ostream & s = cout) const;
  void PrintShort (ostream & s = cout) const;
  int Connect (const string & usersn, bool anonymous,
           int timeout, int debug = 0);
  int Query (Mds::Filter mdsfilter, const string & usersn,
         int timeout, int debug = 0);
  int Result (int timeout, int debug = 0);
  int Find (Mds::Filter mdsfilter, const string & usersn, bool anonymous,
        int timeout, int debug = 0);
 private:
  string name;
  string alias;
  string contact;
  vector <string> support;
  string lrmstype;
  string lrmsversion;
  string lrmsconfig;
  string arch;
  vector <Environment> opsys;
  bool homogeneity;
  string nodecpu;
  int nodemem;
  int totalcpus;
  map <int, int> cpudist;
  long long int sessiondirfree;
  long long int sessiondirtotal;
  long long int cachefree;
  long long int cachetotal;
  vector <Environment> runtimeenv;
  vector <string> localse;
  vector <Environment> middleware;
  int totaljobs;
  int usedcpus;
  int queuedjobs;
  string location;
  vector <string> owner;
  string issuer;
  vector <string> nodeaccess;
  LdapQuery ldap;
 public:
  vector <Queue> queues;
};


/* Target.h */

class Target {

 public:
  Target();
  Target (Cluster * c, Queue * q);
  ~Target () {};
  bool operator! () const;
  bool valid(); // wrapper for operator!
  int CalculateNeededFileSizes (const int timeout);
  long long int GetNeededCacheSize () const;
  long long int GetNeededSessDirSize () const;
  long long int GetNeededTotalSize () const;
  long long int GetRemoteSize () const;
  long long int GetLocalSize () const;
  const Environment * FindEnvironment (const string & attr,
                       const EnvironmentTest & envtest) const;
  int Accept ();
  const string & GetArchitecture () const;
  int GetFrequency () const;
  /* Retruns the maximum allowed running time, -1 is infinite -2 is error */
  int GetMaxCpuTime ();
  int GetCpuTime (int * time) const;
  int GetTotalCpus () const;
  int GetNodeMemory () const;
  int GetQueued () const;
  Cluster * cluster;
  Queue * queue;
  Xrsl xrsl;
 private:
  RemoteFileQuery query;
  long long int neededcachesize;
  long long int neededsessdirsize;
  long long int remotesize;
  long long int localsize;
};


/* Xrsl.h */

class Xrsl {

 public:
  Xrsl (const string & xrslstring);
  Xrsl (const string & xrslfile, int dummy);
  Xrsl (const Xrsl & axrsl);
  Xrsl (globus_rsl_t * axrsl);
  ~Xrsl ();
  Xrsl & operator= (const Xrsl & axrsl);
  bool operator! () const;
  bool valid();    // Swig does not like operator overloading
  Xrsl operator[] (const int ix) const;
  void Print () const;
  const string str () const;
  void AddSimpleRelation (const string & attr, const string & value);
  void AddSimpleRelationSet (const string & attr,
                 const vector <string> & values, bool select);
  int UpdateAttribute (const string & attr, const string & value);
  int Check (globus_rsl_t * axrsl = NULL) const;
  int FixJoin ();
  int FixInOut (const string & stdattr, const string & inoutattr);
  int FixExec ();
  int FixStdin ();
  int FixTime (const string & attr);
  int FixGridTime (const int frequency);
  int FixPeriod (const string & attr);
  int PrepareUpload (vector <string> & filelist);
  int Collect (Target & target, const string & attr,
           globus_rsl_t * axrsl = NULL);
  /* Returns the name of the job - null inidicates error  */
  string GetName ();
  int GetJobName (string & jobname) const;
  /* Retruns the the number of needed cpus -1 is error */
  int GetCpuCount ();
  int GetCount (int * count) const;
  int GetCpuTime (int * time) const;
  int GetGridTime (int * time) const;
  int GetDisk (long long int * disk) const;
  int GetClientXrsl (string & xrslstring) const;
  int GetInputFiles (vector <string> & filelist) const;
  int GetRc (string & rc) const;
  int GetDefaultCache (bool * cache) const;
  void Eval ();
  int Test (Target & target, string & failattr,
        globus_rsl_t * axrsl = NULL) const;
  /* Test the target, "-1" indicates error */
  string TestTarget (Target & target); // wrapper for Test
 private:
  int FindRelation (const string & attr, globus_rsl_t ** relation,
            globus_rsl_t * axrsl = NULL) const;
  globus_list_t ** FindHead ();
  globus_rsl_t * xrsl;
};

/* Giis.h */ 

class Giis {

 public:
  Giis () {};
  Giis (const string & url);
  Giis (const string & host_, int port_, const string & basedn_);
  ~Giis () {};
  bool operator== (const Giis & giis) const;
  bool operator! () const;
  int Connect (const string & usersn, bool anonymous,
           int timeout, int debug = 0);
  int Query (const string & usersn, int timeout, int debug = 0);
  int Result (void callback (const string & attr,
                 const string & value,
                 void * ref),
          void * ref, int timeout, int debug = 0);
 private:
  string host;
  int port;
  string basedn;
  LdapQuery ldap;
};


/* Certinfo.h */

class CertInfo {

public:
  CertInfo (const char * proxy_file = NULL);
  ~CertInfo () {};
  string GetSN () const;
  string GetSNx () const;
  string GetDN () const;
  time_t Expires () const;
  time_t TimeLeft () const;
  bool operator! ();
  bool expired();
private:
  bool good;
  string sn;
  time_t expires;
};

